COMMENT ~

  BEEPER.ASM
  Copyright (c) 1990  Sunny Hill Software  Seattle, WA  USA

  This program relies on the OMNIVIEW external interface to
  illustrate the scheduling patterns and relative quanta of
  OMNIVIEW processes. It does this by sounding a tone, the octave
  number of which is its Console Number, whenever a process
  becomes current.

  The program is a simple, non-removable TSR that must be
  installed prior to running OMNIVIEW. It takes no arguments.

  Keep in mind a few points when listening to this program:

  - The highest priority process, other than the current process,
    will be scheduled at the next opportunity.

  - When all processes have the same priority, the relative
    quantas determine the amount of time each will run.

  - A lower priority process with the same quanta as a higher
    priority process will not run long since the higher priority
    process will intervene at the next scheduling opportunity.

  - Any process in a system with two higher priority processes
    will not run unless at least one of the higher priority
    processes is waiting on a system resource.

  - In a system with default priorities the foreground process
    will run almost exclusively, unless it is waiting on a system
    resource (since it has the highest priority).

  - When a single processes is loaded, and that program is
    waiting, this program will beep continuously as OMNIVIEW
    searches for something that can run.


END COMMENT ~



.Model Tiny



;-- OMNIVIEW external interface functions

USR_INIT       EQU 5400H
; Parameters:
;   es:bx = pointer to locking and device information variables
;   di:dx = pointer to OMNIVIEW system entry

USR_ALLOC      EQU 5401H
; Parameters:
;   es:bx = process handle of allocated process

USR_DEALLOC    EQU 5402H
; Parameters:
;   es:dx = process handle of process to deallocate

USR_SAVE       EQU 5403H
; Parameters:
;   es:dx = process handle of process to save

USR_RESTORE    EQU 5404H
; Parameters:
;   es:dx = process handle of process to restore

USR_FG         EQU 5405H
; Parameters:
;   es:dx = process handle of process to make fg

USR_BG         EQU 5406H
; Parameters:
;   es:dx = process handle of process to make bg

USR_RELEASE    EQU 5407H
; Parameters:
;   NONE


;-- Speaker control macros and equates.

PORT_B           equ 61h
TIMER_2          equ 42h                 ; 8254 Timer channel 2 data port
TIMER_CTL        equ 43h                 ; 8254 Timer mode register data port
TIMER_OUT        equ 01h                 ; PIO port B B0: timer channel 2 output
SPKR_OUT         equ 02h                 ; PIO port B B1: timer out to speaker
SPKR_SWITCH      equ TIMER_OUT+SPKR_OUT  ; need both port B0 & B1 toggle sound

; Allows for TTL logic recovery time on fast CPUs.
;
IO_DELAY macro
    jmp  $+2
endm

; If cnt is specifed moves cnt to AX, pre-loads timer channel 2 with AX to
; produce a periodic sqaure wave (freq = 1,193,180 * AX).
;
; Parms:         cnt or AX = pre-load count
; Result:        void, timer set to generate square wave
; Registers:     AX
;
LOAD_TIMER_CNT   MACRO   cnt
    ifb  <cnt>
         push    ax
    endif
         mov     al, 0B6h                ;; Write timer mode register
         out     TIMER_CTL, al
    ifnb <cnt>
         mov     ax, cnt
    else
         pop     ax
    endif
         out     TIMER_2, al             ;; Write Timer 2 count low byte
         IO_delay
         mov     al, ah
         out     TIMER_2, al             ;; Write timer 2 count high byte
ENDM

; Turns on the speaker control bits in 8255 PIO port B
;
; Parms: void
; Result:        void, speaker turned on
; Registers:     none
;
SPKR_ON          MACRO
         in      al, PORT_B              ;; Get current Port B setting
         IO_delay
         or      al, SPKR_SWITCH         ;; Turn speaker ON
         out     PORT_B, al
ENDM

; Turns off the speaker control bits in 8255 PIO port B
;
; Parms: void
; Result:        void, speaker turned off
; Registers:     none
;
SPKR_OFF MACRO
         in      al, PORT_B              ;; Get current Port B setting
         IO_delay
         and     al, not SPKR_SWITCH     ;; Turn speaker OFF
         out     PORT_B, al
ENDM


; Macro used to call MS-DOS through int 21h with various argument types.
;
MSDOS    MACRO   func
    if   TYPE func EQ 2                  ;; word
         mov     ax, func
    elseif TYPE func EQ 1
         mov     ah, func                ;; byte
    elseif TYPE func EQ 0
         if func GT 0FFh
             if func GT 0FFFFh           ;; word constant
                 .ERR
             else
                 mov     ax, func
             endif
         else
             mov      ah, func           ;; byte constant
         endif
    else
         .ERRNB func
    endif
    int  21h
ENDM



.CODE
     ASSUME    CS:@CurSeg

    org  100h                            ; this is a .COM file
START:
    jmp  main


;-- The external interface code and data structures


null_proc        proc                    ; dummy proc
    ret
null_proc        endp

usr_init_proc    equ     null_proc       ; unused functions
usr_save_proc    equ     null_proc
usr_fg_proc      equ     null_proc
usr_bg_proc      equ     null_proc
usr_release_proc equ     null_proc

proc_handles     dd      10 dup(0)       ; array of process handles

function_table   label   word            ; table of near function handlers
     dw   offset usr_init_proc
     dw   offset usr_alloc_proc
     dw   offset usr_dealloc_proc
     dw   offset usr_save_proc
     dw   offset usr_restore_proc
     dw   offset usr_fg_proc
     dw   offset usr_bg_proc
     dw   offset usr_release_proc

old_int_15 label dword           ; stores old interrupt 15h vector
     old_15_offs dw 0
     old_15_segm dw 0

; The interrupt 15h handler. This filters calls for USR_ALLOC, USR_DEALLOC
; and USR_RESTORE. It tracks all processes in the system and which of them
; is current.
;
int_15_handler proc
         cmp     ax, USR_RESTORE
         ja      int_15_continue
         cmp     ax, USR_ALLOC
         jb      int_15_continue
         cmp     ax, USR_SAVE
         je      int_15_continue

         push    ax                      ; save ax, cx, si, ds
         push    si
         push    ds
         mov     ds, cs:data_segval      ; ds-> data

                                         ; determine function and call it.
         xor     ah, ah
         shl     ax, 1
         mov     si, ax
         mov     si, cs:function_table[si]
         call    si

         pop     ds                      ; restore ax, cx, si, ds
         pop     si
         pop     ax

    int_15_continue:
         jmp  cs:old_int_15              ; jump to old handler

int_15_handler endp


; Records the process handle of the newly created process in the first
; available NULL slot in the proc_handles table (es:bx = proc handle).
;
usr_alloc_proc   proc

         lea     si, proc_handles        ; go forward in proc handle array
         cld

    search:
         lodsw                           ; look for an empty slot
         or      ax, ax
         lodsw
         jnz     search
         or      ax, ax
         jnz     search

         mov     [si][-4], bx            ; record new proc's handle
         mov     ax, es
         mov     [si][-2], ax

         ret                             ; and return
usr_alloc_proc   endp


; Zeros out the proc_handles entry for the process that is being removed from
; the system. (es:dx = process handle)
;
usr_dealloc_proc proc

         push    cx                      ; save cx
         lea     si, proc_handles        ; and go forward in proc handle array
         cld

    desearch:
         lodsw                           ; find the deceased proc in the table
         cmp     ax, dx
         lodsw
         jnz     desearch
         mov     cx, es
         cmp     ax, cx
         jnz     desearch

         xor     ax, ax
         mov     [si][-4], ax            ; zero out the table entry
         mov     [si][-2], ax

         pop     cx                      ; restore cx and return
         ret
usr_dealloc_proc endp


; Matches the process handle of the "just made current" process
; with one of the entries in the process table (es:dx = process
; handle). The index of the entry is the process' Console Number
; and will also serve as the octave number (relative to D#1) of
; the tone used to represent this partition's activity. This proc
; actually turns on the speaker. The speaker is turned off by the
; clock interrupt handler.
;
usr_restore_proc proc

         push    bx                      ; save bx, cx
         push    cx

         lea     si, proc_handles        ; point to handle array
         cld                             ; go forward
         mov     bx, -1                  ; count from zero

    research:
         inc     bx
         lodsw                           ; look for matching process handle
         cmp     ax, dx
         lodsw
         jnz     research
         mov     cx, es
         cmp     ax, cx
         jnz     research

         shl     bx, 1           ; convert index to offset (words)
         mov     ax, count_0[bx] ; get timer preload count for this cons numb
         LOAD_TIMER_CNT          ; set up timer for tone w/ count in AX
         SPKR_ON                 ; connect timer 2 output to speaker amp
         mov     beeping, 1      ; beep for one tick

         pop     cx              ; restore bx, cx
         pop     bx
         ret                     ; and return
usr_restore_proc endp


;-- The timer tick code and data structures


; Old int 08h interrupt vector storage
;
old_clk_vect     label   dword
    old_clk_offs dw      0
    old_clk_segm dw      0


; This is the interrupt 08h handler. It simply decides when it is time to
; turn off the speaker and then does so. The speaker is actually turned on
; by usr_restore_proc
;
clk_int_handler proc

         pushf
         cli
         call    cs:old_clk_vect                 ; call old handler first

         cmp     byte ptr cs:beeping, 0          ; see if anything to do
         jz      Sounded                         ; no
         dec     byte ptr cs:beeping             ; time to turn off speaker?
         jnz     Sounded                         ; no

         push    ax                              ; save AX and turn off speaker
         SPKR_OFF
         xor     ax, ax
         cmp     word ptr cs:tone_count, ax      ; if 0, then we're done
         je      NoSound

         mov     byte ptr cs:beeping, 4          ; else we need to make sure
         mov     cs:tone_count, ax               ; the speaker stays off

    NoSound:
         pop     ax                              ; restore ax

    Sounded:
         iret                                    ; and return
clk_int_handler  endp


;-- Frequency tables, by octaves starting at D#1

CONVERSION_FACTOR        =       1234DCh
FREQ                     =           38
MULT                     =            1

IRPC     counter, 0123456789
    FREQ_&counter&       =       (FREQ * MULT)
    MULT                 =       (MULT * 2)
ENDM

IRPC     counter, 0123456789
    COUNT_&counter&      dw      (CONVERSION_FACTOR / FREQ_&counter&)
ENDM

tone_count       dw      0               ; current tone's preload count
beeping          db      1               ; the duration of the tone/rest
data_segval      dw      0               ; the CSEG relative DGROUP value


;-- end of resident code


MAIN:
         mov     ax, ds                  ; store DGROUP for quick retrieval
         mov     data_segval, ax

         MSDOS   30h                     ; make sure DOS version >=2.0
         cmp     al, 2
         ja      DOS_OK
         int     20h                     ; if not, just exit the old way

    DOS_OK:
         MSDOS   3508h                   ; get old int 8h vector
         mov     old_clk_offs, bx        ; and put it away
         mov     bx, es
         mov     old_clk_segm, bx

         mov     dx, offset clk_int_handler
         MSDOS   2508h                   ; install our int 8h handler

         MSDOS   3515h                   ; get old int 15h vector
         mov     old_15_offs, bx         ; and put it away
         mov     bx, es
         mov     old_15_segm, bx

         mov     dx, offset int_15_handler
         MSDOS   2515h                   ; install our int 15h handler

         mov     dx, offset MAIN         ; drop MAIN
         mov     cx, 4
         shr     dx, cl
         inc     dx
         MSDOS   3100h                   ; and go resident w/ EL = 0

@CurSeg  ends
         end START

